Program prtfile ;
  { Prints a text file on the list device, formatted with various
    user-supplied options.   WPM -- 10/12/84 -- TURBO Pascal }

{$V-}  { Turn off strict type-checking for strings }

    label            99 ;               { for premature exit }

    const
        formfeed   = ^L ;
        bell       = ^G ;
        linelength = 255 ;              { max length of text file lines }

    type
        st_typ  = string[linelength] ;

    var
        line, header               : st_typ ;      { print lines }
        blank_line                 : st_typ ;      { to add indentation }
        page_num, line_cnt, i, n   : integer ;     { counters }
        indent, spacing, max_lines : integer ;     { user-supplied }
        first_page, last_page      : integer ;     { user_supplied }
        fname                      : string[14] ;  { file name }
        ipt_file                   : text ;        { input file }
        ok                         : boolean ;     { whether file exists }
        reply                      : char ;        { to get user response }

    { - - - - - - - - - - - - - - - - }

    procedure print_page_header ;
      { prints header line at top of each page }
        var
            i : integer ;
        begin
            page_num := page_num + 1 ;
            if page_num >= first_page then
              begin
                if page_num > first_page then
                    write (lst, formfeed) ;
                writeln (lst) ;
                write (lst, header) ;
                writeln (lst, page_num) ;
                writeln (lst) ;
                for i := 1 to spacing do
                    writeln (lst)
              end ;
            line_cnt := 3 + spacing
        end ;  { proc print_page_header }

    { - - - - - - - - - - - - - - - - }

    procedure print (line : st_typ ; num_newlines : integer) ;
      { prints a line and the number of newlines indicated }
        var
            i : integer ;
        begin
            if line_cnt > max_lines then
                print_page_header ;
            if page_num >= first_page then
              begin
                write (lst,line) ;
                for i := 1 to num_newlines do
                    writeln (lst)
              end ;
            line_cnt := line_cnt + num_newlines
        end ;  { proc print }

    { - - - - - - - - - - - - - - - - }

    procedure add_blanks (var st : st_typ ; num_blanks : integer) ;
      { appends the number of blanks indicated to the string }
        var
            i : integer ;
        begin
            for i := 1 to num_blanks do
                st := concat (st,' ')
        end ;  { proc add_blanks }

    { - - - - - - - - - - - - - - - - }

    function adjust_line (line : st_typ) : st_typ ;
      { Converts tabs to spaces and adds indentation by moving characters
        one by one from the input string to a work string.  If it encounters
        a tab character it expands the tab to the proper number of spaces.
        Finally, the indentation string is inserted in front of all the
        characters and the function returns the work string. }
        
        const
            tab = ^I ;
        var
            i            : integer ;    { loop counter }
            next_char    : integer ;    { where the next character goes
                                          in the work string }
            work_str     : st_typ ;     { work string to build adjusted line }
        begin
            work_str := '' ;
            next_char := 1 ;
            for i := 1 to length(line) do
                if not (line[i] = tab) then
                  begin
                    work_str := concat(work_str,line[i]) ;
                    next_char := next_char + 1
                  end
                else         { character is a tab -- convert to spaces }
                    repeat
                        work_str := concat(work_str,' ') ;
                        next_char := next_char + 1
                    until (next_char > 8) and ((next_char mod 8) = 1) ;
            insert (blank_line,work_str,1) ;
            adjust_line := work_str
        end ;  { --- proc adjust_line --- }

    { - - - - - - - - - - - - - - - - }

    begin { --- MAIN --- }
        while true do                            { endless loop }
          begin
            writeln ;
            writeln ('This prints a text file, paginated with header.') ;
            writeln ('Please specify options --  <cr> on file name to cancel.') ;
            writeln ('Defaults are no indent, single spacing, 58 lines per page,') ;
            writeln ('start at first page, stop after last.') ;
            writeln ;

            repeat                               { get file name }
                fname := '' ;
                write   ('File name? ') ;
                readln  (fname) ;
                if fname = '' then
                    halt                         { --- Exit loop here --- }
                else
                  begin
                    assign (ipt_file,fname) ;
                    {$i-}
                    reset (ipt_file) ;
                    {$i+}
                    ok := (ioresult = 0) ;
                    if not ok then
                        writeln (bell,'File not found.')
                  end
            until ok ;

            indent := 0 ;                        { get indentation }
            write   ('Number of spaces to indent? ') ;
            readln  (indent) ;
            if indent < 0 then indent := 0 ;
            blank_line := '' ;
            if not (indent = 0 ) then
                for i := 1 to indent do
                    blank_line := concat (' ',blank_line) ;

            spacing := 0 ;                       { get spacing }
            write   ('Line spacing? ') ;
            readln  (spacing) ;
            if spacing < 1 then spacing := 1 ;

            max_lines := 0 ;                     { get page length }
            write   ('Max lines per page? ') ;
            readln  (max_lines) ;
            if max_lines < 1 then
                max_lines := 58 ;

            line := '' ;                         { get header }
            write  ('Header/date? ') ;
            readln (line) ;

            first_page := 0 ;                    { get first page to print }
            write ('Start at what page? ') ;
            readln (first_page) ;
            if first_page < 1 then
                first_page := 1 ;

            last_page := 0 ;                     { get last page to print }
            write ('Quit after what page? ') ;
            readln (last_page) ;
            if last_page < 1 then
                last_page := maxint ;

            header := blank_line ;               { build header line }
            header := concat(header,fname,'  ',line) ;
            if length(header) < 72 then
                add_blanks (header, 72 - length(header))
            else
                add_blanks (header,2) ;
            header := concat (header,'Page ') ;
            page_num := 0 ;
            line_cnt := maxint ;                 { force first page header }

            while not (eof(ipt_file)) do         { print the text file }
              begin
                readln (ipt_file,line) ;
                if not (indent = 0) then         { add identation }
                    line := adjust_line (line) ;
                repeat
                    n := pos(formfeed,line) ;    { handle embedded formfeeds }
                    if not (n = 0) then
                      begin
                        print (copy(line,1,n-1),spacing) ;
                        print_page_header ;
                        delete (line,1,n) ;
                        for i := 1 to indent do
                            line := concat(' ',line) ;
                      end
                until n = 0 ;
                print  (line,spacing) ;

                if keypressed then               { check for premature exit }
                  begin
                    writeln ;
                    write  ('+++ Quit now? (Y/N): ') ;
                    readln (reply) ;
                    if upcase(reply) = 'Y' then
                        goto 99
                  end ;
                if  (page_num = last_page)
                and (line_cnt > max_lines) then
                    goto 99
              end ;

99:         write (lst,formfeed) ;
            writeln (bell,'Done!')
          end
    end.
edded formfeeds }
       